@@ -1,9 +1,9 @@
 # -*- test-case-name: twisted.words.test.test_irc -*-
-# Copyright (c) 2001-2005 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
-"""Internet Relay Chat Protocol for client and server.
+"""
+Internet Relay Chat Protocol for client and server.
 
 Future Plans
 ============
@@ -20,7 +20,7 @@ and \"Transfer #2 is 67% done.\" and otherwise manage the DCC sessions.
 
 Test coverage needs to be better.
 
-@author: U{Kevin Turner<mailto:acapnotic@twistedmatrix.com>}
+@author: Kevin Turner
 
 @see: RFC 1459: Internet Relay Chat Protocol
 @see: RFC 2812: Internet Relay Chat: Client Protocol
@@ -28,30 +28,15 @@ Test coverage needs to be better.
 <http://www.irchelp.org/irchelp/rfc/ctcpspec.html>}
 """
 
-__version__ = '$Revision: 1.94 $'[11:-2]
+import errno, os, random, re, stat, struct, sys, time, types, traceback
+import string, socket
+from os import path
 
 from twisted.internet import reactor, protocol
 from twisted.persisted import styles
 from twisted.protocols import basic
 from twisted.python import log, reflect, text
 
-# System Imports
-
-import errno
-import os
-import random
-import re
-import stat
-import string
-import struct
-import sys
-import time
-import types
-import traceback
-import socket
-
-from os import path
-
 NUL = chr(0)
 CR = chr(015)
 NL = chr(012)
@@ -100,7 +85,13 @@ def split(str, length = 80):
         if w == -1 and n == -1:
             line, str = str[:length], str[length:]
         else:
-            i = n == -1 and w or n
+            if n == -1:
+                i = w
+            else:
+                i = n
+            if i == 0: # just skip the space or newline. don't append any output.
+                str = str[1:]
+                continue
             line, str = str[:i], str[i+1:]
         r.append(line)
     if len(str):
@@ -509,11 +500,11 @@ class IRCClient(basic.LineReceiver):
     @ivar username: Supplied to the server during login as the \"User name\".
         May be C{None}
 
-    @ivar userinfo: Sent in reply to a X{USERINFO} CTCP query.  If C{None}, no
+    @ivar userinfo: Sent in reply to a C{USERINFO} CTCP query.  If C{None}, no
         USERINFO reply will be sent.
         \"This is used to transmit a string which is settable by
         the user (and never should be set by the client).\"
-    @ivar fingerReply: Sent in reply to a X{FINGER} CTCP query.  If C{None}, no
+    @ivar fingerReply: Sent in reply to a C{FINGER} CTCP query.  If C{None}, no
         FINGER reply will be sent.
     @type fingerReply: Callable or String
 
@@ -549,6 +540,26 @@ class IRCClient(basic.LineReceiver):
     dcc_destdir = '.'
     dcc_sessions = None
 
+    # 'mode': (added, removed) i.e.:
+    # 'l': (True, False) accepts an arg when added and no arg when removed
+    # from http://www.faqs.org/rfcs/rfc1459.html - 4.2.3.1 Channel modes
+    # if you want other modes to accept args, add them here, by default unknown
+    # modes won't accept any arg
+    _modeAcceptsArg = {
+        'o': (True, True),    # op/deop a user
+        'h': (True, True),    # hop/dehop (halfop) a user (not defined in RFC)
+        'v': (True, True),    # voice/devoice a user
+        'b': (True, True),    # ban/unban a user/mask
+        'l': (True, False),   # set the user limit to channel
+        'k': (True, False),   # set a channel key (password)
+        't': (False, False),  # only ops set topic
+        's': (False, False),  # secret channel
+        'p': (False, False),  # private channel
+        'i': (False, False),  # invite-only channel
+        'm': (False, False),  # moderated channel
+        'n': (False, False),  # no external messages
+    }
+
     # If this is false, no attempt will be made to identify
     # ourself to the server.
     performLogin = 1
@@ -699,17 +710,22 @@ class IRCClient(basic.LineReceiver):
         self.privmsg(user, channel, message)
 
     def modeChanged(self, user, channel, set, modes, args):
-        """Called when a channel's modes are changed
+        """Called when users or channel's modes are changed.
 
         @type user: C{str}
         @param user: The user and hostmask which instigated this change.
 
         @type channel: C{str}
-        @param channel: The channel for which the modes are changing.
+        @param channel: The channel where the modes are changed. If args is
+        empty the channel for which the modes are changing. If the changes are
+        at server level it could be equal to C{user}.
 
         @type set: C{bool} or C{int}
-        @param set: true if the mode is being added, false if it is being
-        removed.
+        @param set: True if the mode(s) is being added, False if it is being
+        removed. If some modes are added and others removed at the same time
+        this function will be called twice, the first time with all the added
+        modes, the second with the removed ones. (To change this behaviour
+        override the irc_MODE method)
 
         @type modes: C{str}
         @param modes: The mode or modes which are being changed.
@@ -787,7 +803,7 @@ class IRCClient(basic.LineReceiver):
         motd is a list of strings, where each string was sent as a seperate
         message from the server. To display, you might want to use::
 
-            string.join(motd, '\\n')
+            '\\n'.join(motd)
 
         to get a nicely formatted string.
         """
@@ -797,6 +813,15 @@ class IRCClient(basic.LineReceiver):
     ### Your client will want to invoke these.
 
     def join(self, channel, key=None):
+        """
+        Join a channel.
+
+        @type channel: C{str}
+        @param channel: The name of the channel to join. If it has no
+            prefix, C{'#'} will to prepended to it.
+        @type key: C{str}
+        @param key: If specified, the key used to join the channel.
+        """
         if channel[0] not in '&#!+': channel = '#' + channel
         if key:
             self.sendLine("JOIN %s %s" % (channel, key))
@@ -804,6 +829,15 @@ class IRCClient(basic.LineReceiver):
             self.sendLine("JOIN %s" % (channel,))
 
     def leave(self, channel, reason=None):
+        """
+        Leave a channel.
+
+        @type channel: C{str}
+        @param channel: The name of the channel to leave. If it has no
+            prefix, C{'#'} will to prepended to it.
+        @type reason: C{str}
+        @param reason: If given, the reason for leaving.
+        """
         if channel[0] not in '&#!+': channel = '#' + channel
         if reason:
             self.sendLine("PART %s :%s" % (channel, reason))
@@ -811,6 +845,17 @@ class IRCClient(basic.LineReceiver):
             self.sendLine("PART %s" % (channel,))
 
     def kick(self, channel, user, reason=None):
+        """
+        Attempt to kick a user from a channel.
+
+        @type channel: C{str}
+        @param channel: The name of the channel to kick the user from. If it
+            has no prefix, C{'#'} will to prepended to it.
+        @type user: C{str}
+        @param user: The nick of the user to kick.
+        @type reason: C{str}
+        @param reason: If given, the reason for kicking the user.
+        """
         if channel[0] not in '&#!+': channel = '#' + channel
         if reason:
             self.sendLine("KICK %s %s :%s" % (channel, user, reason))
@@ -825,6 +870,12 @@ class IRCClient(basic.LineReceiver):
         If topic is None, then I sent a topic query instead of trying to set
         the topic. The server should respond with a TOPIC message containing
         the current topic of the given channel.
+
+        @type channel: C{str}
+        @param channel: The name of the channel to change the topic on. If it
+            has no prefix, C{'#'} will to prepended to it.
+        @type topic: C{str}
+        @param topic: If specified, what to set the topic to.
         """
         # << TOPIC #xtestx :fff
         if channel[0] not in '&#!+': channel = '#' + channel
@@ -834,7 +885,28 @@ class IRCClient(basic.LineReceiver):
             self.sendLine("TOPIC %s" % (channel,))
 
     def mode(self, chan, set, modes, limit = None, user = None, mask = None):
-        """Change the modes on a user or channel."""
+        """
+        Change the modes on a user or channel.
+
+        The C{limit}, C{user}, and C{mask} parameters are mutually exclusive.
+
+        @type chan: C{str}
+        @param chan: The name of the channel to operate on. If it has no
+            prefix, C{'#'} will to prepended to it.
+        @type set: C{bool}
+        @param set: True to give the user or channel permissions and False to
+            remove them.
+        @type modes: C{str}
+        @param modes: The mode flags to set on the user or channel.
+        @type limit: C{int}
+        @param limit: In conjuction with the C{'l'} mode flag, limits the
+             number of users on the channel.
+        @type user: C{str}
+        @param user: The user to change the mode on.
+        @type mask: C{str}
+        @param mask: In conjuction with the C{'b'} mode flag, sets a mask of
+            users to be banned from the channel.
+        """
         if set:
             line = 'MODE %s +%s' % (chan, modes)
         else:
@@ -849,9 +921,25 @@ class IRCClient(basic.LineReceiver):
 
 
     def say(self, channel, message, length = None):
+        """
+        Send a message to a channel
+
+        @type channel: C{str}
+        @param channel: The channel to say the message on.
+        @type message: C{str}
+        @param message: The message to say.
+        @type length: C{int}
+        @param length: The maximum number of octets to send at a time.  This
+            has the effect of turning a single call to C{msg()} into multiple
+            commands to the server.  This is useful when long messages may be
+            sent that would otherwise cause the server to kick us off or
+            silently truncate the text we are sending.  If None is passed, the
+            entire message is always send in one command.
+        """
         if channel[0] not in '&#!+': channel = '#' + channel
         self.msg(channel, message, length)
 
+
     def msg(self, user, message, length = None):
         """Send a message to a user or channel.
 
@@ -888,12 +976,64 @@ class IRCClient(basic.LineReceiver):
                 lines)
 
     def notice(self, user, message):
+        """
+        Send a notice to a user.
+
+        Notices are like normal message, but should never get automated
+        replies.
+
+        @type user: C{str}
+        @param user: The user to send a notice to.
+        @type message: C{str}
+        @param message: The contents of the notice to send.
+        """
         self.sendLine("NOTICE %s :%s" % (user, message))
 
     def away(self, message=''):
+        """
+        Mark this client as away.
+
+        @type message: C{str}
+        @param message: If specified, the away message.
+        """
         self.sendLine("AWAY :%s" % message)
 
+
+
+    def back(self):
+        """
+        Clear the away status.
+        """
+        # An empty away marks us as back
+        self.away()
+
+
+    def whois(self, nickname, server=None):
+        """
+        Retrieve user information about the given nick name.
+
+        @type nickname: C{str}
+        @param nickname: The nick name about which to retrieve information.
+
+        @since: 8.2
+        """
+        if server is None:
+            self.sendLine('WHOIS ' + nickname)
+        else:
+            self.sendLine('WHOIS %s %s' % (server, nickname))
+
+
     def register(self, nickname, hostname='foo', servername='bar'):
+        """
+        Login to the server.
+
+        @type nickname: C{str}
+        @param nickname: The nickname to register.
+        @type hostname: C{str}
+        @param hostname: If specified, the hostname to logon as.
+        @type servername: C{str}
+        @param servername: If specified, the servername to logon as.
+        """
         if self.password is not None:
             self.sendLine("PASS %s" % self.password)
         self.setNick(nickname)
@@ -902,16 +1042,37 @@ class IRCClient(basic.LineReceiver):
         self.sendLine("USER %s %s %s :%s" % (self.username, hostname, servername, self.realname))
 
     def setNick(self, nickname):
+        """
+        Set this client's nickname.
+
+        @type nickname: C{str}
+        @param nickname: The nickname to change to.
+        """
         self.nickname = nickname
         self.sendLine("NICK %s" % nickname)
 
     def quit(self, message = ''):
+        """
+        Disconnect from the server
+
+        @type message: C{str}
+
+        @param message: If specified, the message to give when quitting the
+            server.
+        """
         self.sendLine("QUIT :%s" % message)
 
     ### user input commands, client->client
 
     def me(self, channel, action):
-        """Strike a pose.
+        """
+        Strike a pose.
+
+        @type channel: C{str}
+        @param channel: The name of the channel to have an action on. If it
+            has no prefix, C{'#'} will to prepended to it.
+        @type action: C{str}
+        @param action: The action to preform.
         """
         if channel[0] not in '&#!+': channel = '#' + channel
         self.ctcpMakeQuery(channel, [('ACTION', action)])
@@ -920,7 +1081,8 @@ class IRCClient(basic.LineReceiver):
     _MAX_PINGRING = 12
 
     def ping(self, user, text = None):
-        """Measure round-trip delay to another IRC client.
+        """
+        Measure round-trip delay to another IRC client.
         """
         if self._pings is None:
             self._pings = {}
@@ -983,15 +1145,27 @@ class IRCClient(basic.LineReceiver):
     ### but it is safe to leave them alone.
 
     def irc_ERR_NICKNAMEINUSE(self, prefix, params):
+        """
+        Called when we try to register an invalid nickname.
+        """
         self.register(self.nickname+'_')
 
     def irc_ERR_PASSWDMISMATCH(self, prefix, params):
+        """
+        Called when the login was incorrect.
+        """
         raise IRCPasswordMismatch("Password Incorrect.")
 
     def irc_RPL_WELCOME(self, prefix, params):
+        """
+        Called when we have received the welcome from the server.
+        """
         self.signedOn()
 
     def irc_JOIN(self, prefix, params):
+        """
+        Called when a user joins a channel.
+        """
         nick = string.split(prefix,'!')[0]
         channel = params[-1]
         if nick == self.nickname:
@@ -1000,6 +1174,9 @@ class IRCClient(basic.LineReceiver):
             self.userJoined(nick, channel)
 
     def irc_PART(self, prefix, params):
+        """
+        Called when a user leaves a channel.
+        """
         nick = string.split(prefix,'!')[0]
         channel = params[0]
         if nick == self.nickname:
@@ -1008,20 +1185,59 @@ class IRCClient(basic.LineReceiver):
             self.userLeft(nick, channel)
 
     def irc_QUIT(self, prefix, params):
+        """
+        Called when a user has quit.
+        """
         nick = string.split(prefix,'!')[0]
         self.userQuit(nick, params[0])
 
+
     def irc_MODE(self, prefix, params):
-        channel, rest = params[0], params[1:]
-        set = rest[0][0] == '+'
-        modes = rest[0][1:]
-        args = rest[1:]
-        self.modeChanged(prefix, channel, set, modes, tuple(args))
+        """
+        Parse the server message when one or more modes are changed
+        """
+        user, channel, modes, args = prefix, params[0], params[1], params[2:]
+        if modes[0] not in '+-':
+            # add a '+' before the modes if it isn't specified (e.g. MODE s)
+            modes = '+' + modes
+        if ((modes[0] == '+' and '-' not in modes[1:]) or
+            (modes[0] == '-' and '+' not in modes[1:])):
+            # all modes are added or removed
+            set = (modes[0] == '+')
+            modes = modes[1:].replace('-+'[set], '')
+            self.modeChanged(user, channel, set, modes, tuple(args))
+        else:
+            # some modes added and other removed
+            modes2, args2 = ['', ''], [[], []]
+            for c in modes:
+                if c == '+':
+                    i = 0
+                elif c == '-':
+                    i = 1
+                else:
+                    modes2[i] += c
+                    # take an arg only if the mode accepts it (e.g. +o nick)
+                    if args and self._modeAcceptsArg.get(c, (False, False))[i]:
+                        args2[i].append(args.pop(0))
+            if args:
+                log.msg('Too many args (%s) received for %s. If one or more '
+                    'modes are supposed to accept an arg and they are not in '
+                    '_modeAcceptsArg, add them.' % (' '.join(args), modes))
+
+            self.modeChanged(user, channel, True, modes2[0], tuple(args2[0]))
+            self.modeChanged(user, channel, False, modes2[1], tuple(args2[1]))
+
 
     def irc_PING(self, prefix, params):
+        """
+        Called when some has pinged us.
+        """
         self.sendLine("PONG %s" % params[-1])
 
     def irc_PRIVMSG(self, prefix, params):
+        """
+        Called when we get a message.
+        """
         user = prefix
         channel = params[0]
         message = params[-1]
@@ -1041,6 +1257,9 @@ class IRCClient(basic.LineReceiver):
         self.privmsg(user, channel, message)
 
     def irc_NOTICE(self, prefix, params):
+        """
+        Called when a user gets a notice.
+        """
         user = prefix
         channel = params[0]
         message = params[-1]
@@ -1058,6 +1277,9 @@ class IRCClient(basic.LineReceiver):
         self.noticed(user, channel, message)
 
     def irc_NICK(self, prefix, params):
+        """
+        Called when a user changes their nickname.
+        """
         nick = string.split(prefix,'!', 1)[0]
         if nick == self.nickname:
             self.nickChanged(params[0])
@@ -1065,7 +1287,8 @@ class IRCClient(basic.LineReceiver):
             self.userRenamed(nick, params[0])
 
     def irc_KICK(self, prefix, params):
-        """Kicked?  Who?  Not me, I hope.
+        """
+        Called when a user is kicked from a channel.
         """
         kicker = string.split(prefix,'!')[0]
         channel = params[0]
@@ -1078,7 +1301,8 @@ class IRCClient(basic.LineReceiver):
             self.userKicked(kicked, channel, kicker, message)
 
     def irc_TOPIC(self, prefix, params):
-        """Someone in the channel set the topic.
+        """
+        Someone in the channel set the topic.
         """
         user = string.split(prefix, '!')[0]
         channel = params[0]
@@ -1086,7 +1310,9 @@ class IRCClient(basic.LineReceiver):
         self.topicUpdated(user, channel, newtopic)
 
     def irc_RPL_TOPIC(self, prefix, params):
-        """I just joined the channel, and the server is telling me the current topic.
+        """
+        Called when the topic for a channel is initially reported or when it
+        subsequently changes.
         """
         user = string.split(prefix, '!')[0]
         channel = params[1]
@@ -1399,7 +1625,8 @@ class IRCClient(basic.LineReceiver):
                  % (user, tag, data))
 
     def ctcpMakeReply(self, user, messages):
-        """Send one or more X{extended messages} as a CTCP reply.
+        """
+        Send one or more C{extended messages} as a CTCP reply.
 
         @type messages: a list of extended messages.  An extended
         message is a (tag, data) tuple, where 'data' may be C{None}.
@@ -1409,7 +1636,8 @@ class IRCClient(basic.LineReceiver):
     ### client CTCP query commands
 
     def ctcpMakeQuery(self, user, messages):
-        """Send one or more X{extended messages} as a CTCP query.
+        """
+        Send one or more C{extended messages} as a CTCP query.
 
         @type messages: a list of extended messages.  An extended
         message is a (tag, data) tuple, where 'data' may be C{None}.
@@ -1420,7 +1648,8 @@ class IRCClient(basic.LineReceiver):
     ### You may want to add methods here, or override UnknownReply.
 
     def ctcpReply(self, user, channel, messages):
-        """Dispatch method for any CTCP replies received.
+        """
+        Dispatch method for any CTCP replies received.
         """
         for m in messages:
             method = getattr(self, "ctcpReply_%s" % m[0], None)
